@@ -1,23 +1,27 @@
 """
     Primary Guard hook code
 """
-from typing import Optional, MutableMapping, Any
 import json
 import logging
 from dataclasses import asdict
 from types import ModuleType
-from jinja2 import Environment, FileSystemLoader, select_autoescape
+from typing import Any, Dict, Iterator, List, MutableMapping, Optional, Union
+
+import cfn_guard_rs
+from cfn_guard_rs.errors import GuardError, MissingValueError, ParseError
 from cloudformation_cli_python_lib import (
-    Hook,
     BaseHookHandlerRequest,
-    ProgressEvent,
-    SessionProxy,
+    HandlerErrorCode,
+    Hook,
     HookInvocationPoint,
     OperationStatus,
-    HandlerErrorCode,
+    ProgressEvent,
+    SessionProxy,
 )
-import cfn_guard_rs
-from cfn_guard_rs.errors import (GuardError, ParseError, MissingValueError)
+from jinja2 import Environment, FileSystemLoader, select_autoescape
+from jsonpath_rw import DatumInContext, parse
+
+from cfn_guard_rs_hook.types import Converter
 
 LOG = logging.getLogger(__name__)
 
@@ -33,6 +37,7 @@ class GuardHook(Hook):
         type_name: str,
         type_configuration: Any,
         rules: ModuleType,
+        converters: Optional[List[Converter]] = None,
     ) -> None:
         super().__init__(type_name, type_configuration)
         self._handlers = {
@@ -44,6 +49,10 @@ class GuardHook(Hook):
             loader=FileSystemLoader(rules.__path__), autoescape=select_autoescape()
         )
         self.rules = rules
+        if converters is not None:
+            self.converters = converters
+        else:
+            self.converters = []
 
     # pylint: disable=unused-argument
     def pre_create_handler(
@@ -121,7 +130,7 @@ class GuardHook(Hook):
                 and request.hookContext.targetLogicalId
                 and request.hookContext.targetName
             ):
-                template = self.__make_cloudformation(
+                template = self._make_cloudformation(
                     target_model.get("resourceProperties", {}),
                     request.hookContext.targetLogicalId,
                     request.hookContext.targetName,
@@ -144,7 +153,7 @@ class GuardHook(Hook):
             progress.status = OperationStatus.FAILED
             progress.errorCode = HandlerErrorCode.InternalFailure
             progress.message = str(err)
-        except Exception as err:  #pylint: disable=broad-except
+        except Exception as err:  # pylint: disable=broad-except
             # Catch all other types of errors
             LOG.error(err)
             progress.status = OperationStatus.FAILED
@@ -153,7 +162,49 @@ class GuardHook(Hook):
 
         return progress
 
-    def __make_cloudformation(
+    def _get_path(self, match: DatumInContext):
+        """Return an iterator based upon MATCH.PATH. Each item is a path component,
+        start from outer most item.
+
+        Args:
+            match;
+        Returns:
+            dict: returns a value which could be a list, Dict, str, etc.
+        """
+        if match.context is not None:
+            for path_element in self._get_path(match.context):
+                yield path_element
+            yield str(match.path)
+
+    def _update_json(self, obj: Dict, path: Iterator[str], value: Any) -> Any:
+        """Update JSON Schema 'path' on 'obj' with 'value'
+
+        Args:
+            obj (Dict): the object that we will look for the path in
+            path (Iterator[str]): the path that we are iterating down
+            value (Any): the value to assign the end of the path
+        Returns:
+            dict: returns a value which could be a list, Dict, str, etc.
+        """
+        first: Union[str, int]
+        try:
+            first = next(path)
+            # check if item is an array.  Example: [0]
+            # we validate if its an array and then we get the int value in
+            # the middle
+            if first.startswith("[") and first.endswith("]"):
+                try:
+                    first = int(first[1:-1])
+                except ValueError:
+                    pass
+            # Update the json object with the returned valued
+            obj[first] = self._update_json(obj[first], path, value)
+            return obj
+        except StopIteration:
+            # Represents that next reached the end so we return the value
+            return value
+
+    def _make_cloudformation(
         self, props: dict, resource_name: str, resource_type: str
     ) -> dict:
         """
@@ -176,6 +227,19 @@ class GuardHook(Hook):
         dict
             A valid CloudFormation template
         """
+
+        for converter in self.converters:
+            # parse the JSON Path into a JSON path expression
+            jsonpath_expr = parse(converter.path)
+            # find all matches for the JSON Path.  There are multiples
+            # because of * or looking inside of lists of objects
+            matches = jsonpath_expr.find(props)
+            for match in matches:
+                # for each converter we need to update the value as appropriate
+                value = converter.converter(match.value)
+                # Update JSON with the path from the match and new value
+                props = self._update_json(props, self._get_path(match), value)
+
         return {
             "Resources": {resource_name: {"Type": resource_type, "Properties": props}}
         }
