@@ -22,7 +22,7 @@ def to_int(value: str) -> int:
 
 def to_bool(value: str) -> bool:
     """Convert string to bool"""
-    return bool(value)
+    return value.lower() == "true"
 
 
 def to_float(value: str) -> float:
